@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-// +build darwin dragonfly freebsd linux nacl netbsd openbsd solaris windows
+// +build aix darwin dragonfly freebsd hurd js,wasm linux netbsd openbsd solaris windows
 
 package signal
 
@@ -15,6 +15,7 @@ import (
 func signal_disable(uint32)
 func signal_enable(uint32)
 func signal_ignore(uint32)
+func signal_ignored(uint32) bool
 func signal_recv() uint32
 
 func loop() {
@@ -24,8 +25,7 @@ func loop() {
 }
 
 func init() {
-	signal_enable(0) // first call - initialize
-	go loop()
+	watchSignalLoop = loop
 }
 
 const (
@@ -56,3 +56,7 @@ func disableSignal(sig int) {
 func ignoreSignal(sig int) {
 	signal_ignore(uint32(sig))
 }
+
+func signalIgnored(sig int) bool {
+	return signal_ignored(uint32(sig))
+}
