@@ -4,10 +4,15 @@
 
 package runtime
 
+import (
+	"internal/bytealg"
+	_ "unsafe" // for go:linkname
+)
+
 // Frames may be used to get function/file/line information for a
 // slice of PC values returned by Callers.
 type Frames struct {
-	// callers is a slice of PCs that have not yet been expanded.
+	// callers is a slice of PCs that have not yet been expanded to frames.
 	callers []uintptr
 
 	// The last PC we saw.
@@ -79,10 +84,15 @@ func (ci *Frames) Next() (frame Frame, more bool) {
 
 	// Subtract 1 from PC to undo the 1 we added in callback in
 	// go-callers.c.
-	function, file, line := funcfileline(pc-1, int32(i))
+	function, file, line, _ := funcfileline(pc-1, int32(i), more)
 	if function == "" && file == "" {
 		return Frame{}, more
 	}
+
+	// Demangle function name if needed.
+	function = demangleSymbol(function)
+
+	// Create entry.
 	entry := funcentry(pc - 1)
 	f := &Func{name: function, entry: entry}
 
@@ -103,6 +113,33 @@ func (ci *Frames) Next() (frame Frame, more bool) {
 	return frame, more
 }
 
+//go:noescape
+// pcInlineCallers is written in C.
+func pcInlineCallers(pc uintptr, locbuf *location, max int32) int32
+
+// runtime_expandFinalInlineFrame expands the final pc in stk to include all
+// "callers" if pc is inline.
+//
+//go:linkname runtime_expandFinalInlineFrame runtime_1pprof.runtime__expandFinalInlineFrame
+func runtime_expandFinalInlineFrame(stk []uintptr) []uintptr {
+	if len(stk) == 0 {
+		return stk
+	}
+	pc := stk[len(stk)-1]
+	tracepc := pc - 1
+
+	var locbuf [_TracebackMaxFrames]location
+	n := pcInlineCallers(tracepc, &locbuf[0], int32(len(locbuf)))
+
+	// Returning the same PC several times causes Frame.Next to do
+	// the right thing.
+	for i := int32(1); i < n; i++ {
+		stk = append(stk, pc)
+	}
+
+	return stk
+}
+
 // NOTE: Func does not expose the actual unexported fields, because we return *Func
 // values to users, and we want to keep them from being able to overwrite the data
 // with (say) *f = Func{}.
@@ -119,9 +156,10 @@ type Func struct {
 // given program counter address, or else nil.
 //
 // If pc represents multiple functions because of inlining, it returns
-// the *Func describing the outermost function.
+// the *Func describing the innermost function, but with an entry of
+// the outermost function.
 func FuncForPC(pc uintptr) *Func {
-	name, _, _ := funcfileline(pc, -1)
+	name, _, _, _ := funcfileline(pc, -1, false)
 	if name == "" {
 		return nil
 	}
@@ -150,10 +188,104 @@ func (f *Func) Entry() uintptr {
 // The result will not be accurate if pc is not a program
 // counter within f.
 func (f *Func) FileLine(pc uintptr) (file string, line int) {
-	_, file, line = funcfileline(pc, -1)
+	_, file, line, _ = funcfileline(pc, -1, false)
 	return file, line
 }
 
+func hexval(b byte) uint {
+	if b >= '0' && b <= '9' {
+		return uint(b - '0')
+	}
+	if b >= 'a' && b <= 'f' {
+		return uint(b-'a') + 10
+	}
+	return 0
+}
+
+func hexDigitsToRune(digits []byte, ndig int) rune {
+	result := uint(0)
+	for i := 0; i < ndig; i++ {
+		result <<= uint(4)
+		result |= hexval(digits[i])
+	}
+	return rune(result)
+}
+
+// decodeIdentifier performs an in-place decoding on the input byte slice.
+// This undoes the compiler underscore mangling.
+// Returns the number of bytes used by the result.
+func decodeIdentifier(bsl []byte) int {
+	underscoreCodes := map[byte]byte{
+		'_': '_',
+		'0': '.',
+		'1': '/',
+		'2': '*',
+		'3': ',',
+		'4': '{',
+		'5': '}',
+		'6': '[',
+		'7': ']',
+		'8': '(',
+		'9': ')',
+		'a': '"',
+		'b': ' ',
+		'c': ';',
+	}
+
+	j := 0
+	for i := 0; i < len(bsl); i++ {
+		b := bsl[i]
+		if b != '_' || i+1 >= len(bsl) {
+			bsl[j] = b
+			j++
+			continue
+		}
+
+		if d, ok := underscoreCodes[bsl[i+1]]; ok {
+			i++
+			bsl[j] = d
+			j++
+			continue
+		}
+
+		rlen := 0
+		switch bsl[i+1] {
+		case 'x':
+			rlen = 2
+		case 'u':
+			rlen = 4
+		case 'U':
+			rlen = 8
+		}
+
+		if rlen > 0 && i+1+rlen < len(bsl) {
+			r := hexDigitsToRune(bsl[i+2:], rlen)
+			nc := encoderune(bsl[j:], r)
+			j += nc
+			i += rlen + 1
+		} else {
+			bsl[j] = b
+			j++
+		}
+	}
+	return j
+}
+
+// Demangle a function symbol. Applies the reverse of go_encode_id()
+// as used in the compiler.
+
+func demangleSymbol(s string) string {
+	if bytealg.IndexByteString(s, '.') < 0 {
+		// A symbol with no '.' is not a Go symbol.
+		return s
+	}
+
+	bsl := []byte(s)
+	nchars := decodeIdentifier(bsl)
+	bsl = bsl[:nchars]
+	return string(bsl)
+}
+
 // implemented in go-caller.c
-func funcfileline(uintptr, int32) (string, string, int)
+func funcfileline(uintptr, int32, bool) (string, string, int, int)
 func funcentry(uintptr) uintptr
